--[[
	F3F Practice

	Telemetry script
	REQUIRES OpenTX 2.0.13+

	Provides a countdown of 30 seconds followed by a chronometer for F3F practice
	NB: there's no telemetry at all in this "telemetry" script.

	Releases
	1.0 Initial release

	Pierre Rondel
--]]


local CHRONO_SWITCH = getFieldInfo( 'sg' ).id 	-- a 3-pos switch is mandatory here
local TMP_SWITCH = getFieldInfo( 'sh' ).id 	-- temporary switch to reset the task
local SOUND_PATH = '/SOUNDS/EN/'
local times	-- time keeper object
local timer1	-- 30s to gain altitude
local timer2	-- F3F flight time
local state 	-- 0=inactive, 1=countdown in progress, 2=run in progress
local played30s
local played25s
local played20s
local played15s
local played10s
local played5s
local played4s
local played3s
local played2s
local played1s
local finalTimePlayed
local intro
local finalTime
local bestTime 



-- Wrapper around the timer interface
--
-- countdownBeep 	integer (none, beep, voice)
-- minuteBeep		bool
-- persistent		integer (none, flight, manual reset)
--
local function createTimer( timerId, startValue, countdownBeep, minuteBeep, persistent )

	-- Precondition: timerId is either 0 or 1
	local id = timerId
	local timer = model.getTimer( id )
	
	local function getVal()
		timer.value = model.getTimer( id ).value
		return timer.value
	end

	local function start()
		timer.mode = 1
		model.setTimer( id, timer )
	end

	local function stop()
		timer = model.getTimer( id )
		local running = (timer.mode > 0)
		timer.mode = 0
		model.setTimer( id, timer )
		return running, timer.value
	end

	local function reset()
		-- Really don't know how it's supposed to work, as there's no doc, so, let's be conservative...
		-- stop it first (should already be done)
		timer.mode = 0
		model.setTimer( id, timer )

		-- reset it and resync our state with whatever resetTimer() did
		model.resetTimer( id )
		timer = model.getTimer( id )

		-- set the start value back to the original starting value in case it was changed
		timer.start = startValue
		timer.value = startValue
		model.setTimer( id, timer )
	end


	local function drawImpl( x, y, val, att )
		lcd.drawTimer( x, y, val, att )
		return val
	end

	local function draw( x, y, att )
		return drawImpl( x, y, getVal(), att )
		
	end


	-- "constructor"
	if countdownBeep ~= null then
		timer.countdownBeep = countdownBeep
	end

	if minuteBeep ~= null then
		timer.minuteBeep = minuteBeep
	end

	if persistent ~= null then
		timer.persistent = persistent
	end

	if startValue then
		timer.value = startValue
		timer.start = startValue
	end

	timer.mode = 0
	model.setTimer( id, timer )


	return {
		start = start,
		stop = stop,
		reset = reset,
		draw = draw,
		getVal = getVal,
	}
end

-- Time vocal announcement

local function playTime( time )
	local val = math.floor( time / 60 )
	if val > 0 then
		playNumber( val, 16, 0 )	-- 16 = minutes
	end
	val = time % 60
	if val > 0 then
		playNumber( val, 17, 0 )	-- 17 = seconds
	end
end


--
--  Time Keeper 
--    * Record several flight times and display them on the right side of the screen
--

function createTimeKeeper( size )
	local tab = {}
	
	local function initialize( size )
		for i=1,size do
			tab[ i ] = 0
		end
	end

	-- stacks time, discarding the older values
	local function pushTime( t )
		for i=2,6 do
			tab[ i-1 ] = tab[ i ]
		end
		tab[6] = t
	end

	local function getVal( i )
		-- Precondition : 1 <= i <= (#tab - 1)
		return tab[ i ]
	end

	local function reset()
		initialize( 6 )
	end

	local function draw( x, y, i, att )
		local val = tab[ i ]
		if val > 0 then
			lcd.drawTimer( x, y, val, att )
		else 
			lcd.drawText( x, y, '--:--', att )
		end
	end


	-- "constructor"
	initialize( size + 1 )

	return { 
		pushTime=pushTime,
		getVal=getVal,
		reset=reset,
		draw=draw
	}
end


local function drawTimeKeeper()

		for i=1,6 do
			times.draw( 175, 5+(i-1)*9, i, 0 )
		end
end



local function drawCountdown()

	timer1.draw( 24, 4, XXLSIZE )
	lcd.drawText( lcd.getLastPos(), 4, "S", DBLSIZE )
	-- lcd.drawText( 135, 27, "S", DBLSIZE )
end

local function drawChronometer()

	timer2.draw( 24, 4, XXLSIZE )
	lcd.drawText( lcd.getLastPos(), 4, "S", DBLSIZE )
	-- lcd.drawText( 135, 27, "S", DBLSIZE )
end

local function drawFrame()

	lcd.drawLine( 0, 47, 159, 47, SOLID, 2 )
	lcd.drawPoint( 0, 47 )
	lcd.drawLine( 159, 0, 159, 63, SOLID, 2 )
	lcd.drawPoint( 159, 0 )
end


--
-- FUNCTION INIT()
--
--

local function init()

	
	-- create a time keeper to record times
	times = createTimeKeeper(6)
	
	-- createTime parameters : timerId, startValue, countdownBeep, minuteBeep, persistent
	timer1 = createTimer( 0, 30, 0, false, 0 )	-- 30s to gain altitude
	timer2 = createTimer( 1, 0, 0, false, 0 )       -- current flight time 

	drawCountdown()
	state = 0 

	played30s = 0
	played25s = 0
	played20s = 0
	played15s = 0
	played10s = 0
	played5s = 0
 	played4s = 0
	played3s = 0
	played2s = 0
	played1s = 0

	finalTimePlayed = 0
	intro = 0
	finalTime = 0
	bestTime = 0
end


--
-- FUNCTION RUN()
-- Main function which is active when the telemetry screen is displayed
--

local function run()

	-- play the vocal intro only once
	if intro == 0 then
		-- vocal annoucement "welcome to F3F practice"
		playFile( SOUND_PATH .. 'welcome.wav' )
		intro = 1
	end

	drawFrame()
	drawTimeKeeper()
	drawCountdown()
			
	lcd.drawText( 27, 53, '   BEST TIME :   ', INVERS )
	if bestTime > 0 then lcd.drawTimer( 100, 53, bestTime, INVERS )
	else lcd.drawText( 100, 53, '--:--', INVERS )
	end
	
	-- start or restart the countdown using the temporary switch
	if state < 2  and getValue( CHRONO_SWITCH ) < 0 and getValue( TMP_SWITCH ) >= 0 then

		-- re-initialise timers
		timer1.reset()
		timer2.reset()

		-- Start countdown
		timer1.start()
		drawCountdown()

		state = 1	
		
		played25s = 0
		played20s = 0
		played15s = 0
		played10s = 0
		played5s = 0
		played4s = 0
		played3s = 0
		played2s = 0
		played1s = 0
		finalTimePlayed = 0
		

		if played30s == 0 then 
			playFile( SOUND_PATH .. '30seconds.wav' )
			played30s = 1
		end
	end

	-- countdown normally in progress
	if state == 1 and getValue( CHRONO_SWITCH ) < 0 and getValue( TMP_SWITCH ) < 0 then

		-- but late start as countdown has finished
		if timer1.getVal() <= 0 then   

			-- This is the case, so stop the countdown of 30s
			timer1.stop()

			-- Start the chronometer 
			timer2.start()	
			drawChronometer()

			-- Play a beep to indicate that the chonometer started
			playFile( SOUND_PATH .. 'beep-02.wav' )
			playFile( SOUND_PATH .. 'start.wav' )

			state = 2
		end
	

		-- countdown continues

		drawCountdown()

		if timer1.getVal() == 25 and played25s == 0 then 
			playFile( SOUND_PATH .. '25seconds.wav' ) 
			played25s = 1 
		end

		if timer1.getVal() == 20 and played20s == 0 then 
			playFile( SOUND_PATH .. '20seconds.wav' ) 
			played20s = 1 
		end

		if timer1.getVal() == 15 and played15s == 0 then playFile( SOUND_PATH .. '15seconds.wav' ) played15s = 1 end
		if timer1.getVal() == 10 and played10s == 0 then playFile( SOUND_PATH .. '10seconds.wav' ) played10s = 1 end
		if timer1.getVal() == 5  and played5s == 0  then playFile( SOUND_PATH .. '5seconds.wav' )  played5s = 1 end
		if timer1.getVal() == 4  and played4s == 0  then playFile( SOUND_PATH .. '4seconds.wav' )  played4s = 1 end
		if timer1.getVal() == 3  and played3s == 0  then playFile( SOUND_PATH .. '3seconds.wav' )  played3s = 1 end
		if timer1.getVal() == 2  and played2s == 0  then playFile( SOUND_PATH .. '2seconds.wav' )  played2s = 1 end
		if timer1.getVal() == 1  and played1s == 0  then playFile( SOUND_PATH .. '1seconds.wav' )  played1s = 1 end	
	end	
	

	-- Early start (countdown not finished, but F3F run started)
	if state == 1 and getValue( CHRONO_SWITCH ) == 0 and getValue( TMP_SWITCH ) < 0  then

		-- This is the case, so stop the countdown of 30s
		timer1.stop()
		timer1.reset()

		-- Start the chronometer 
		timer2.start()	
		drawChronometer()

		-- Play a beep to indicate that the chonometer started
		playFile( SOUND_PATH .. 'beep-02.wav' )
		playFile( SOUND_PATH .. 'start.wav' )

		state = 2
	end

	-- F3F run in progress, nothing to do except display the chronometer
	if state == 2 and getValue( CHRONO_SWITCH ) == 0 then
		
		drawChronometer()

	end
	

	-- F3F run finished, as chronometer has been stopped (which can be done only manually)
	if state == 2 and getValue( CHRONO_SWITCH ) > 0 then

		-- display the chronometer
		-- lcd.drawLine( 159, 41, 211, 41, SOLID, 2 )
		drawChronometer()

		-- stop the chronometer
		timer2.stop()

		-- do a vocal announcement of the time
		
		if finalTimePlayed == 0 then 

			finalTime = timer2.getVal()
			playTime(timer2.getVal()) 
			finalTimePlayed = 1

			-- push the time in the Time Keeper
			times.pushTime(finalTime)	
			drawTimeKeeper()
		end

		-- compare to the best time
		if bestTime == 0 then bestTime = finalTime end
		if bestTime > 0 and finalTime < bestTime then
		 	bestTime = finalTime
		end	

		lcd.drawText(0, 53, '                        ', 0)
		lcd.drawText( 27, 53, '   BEST TIME :   ', INVERS )
		if bestTime > 0 then lcd.drawTimer( 100, 53, bestTime, INVERS )
		else lcd.drawText( 100, 53, '--:--', INVERS )
		end
	end	


	-- start a new F3F run, after a RESET using the temporary switch
	if state == 2 and getValue( CHRONO_SWITCH ) < 0 and getValue( TMP_SWITCH ) >= 0 then

		-- re-initialise timers
		timer1.reset()
		timer2.reset()

		-- Start countdown
		timer1.start()
		drawCountdown()
		playFile( SOUND_PATH .. '30seconds.wav' )
		played30s = 1

		state = 1
		played25s = 0
		played20s = 0
		played15s = 0
		played10s = 0
		played5s = 0
		played4s = 0
		played3s = 0
		played2s = 0
		played1s = 0
		finalTimePlayed = 0
	end

	if state == 2 and getValue( CHRONO_SWITCH ) < 0 and getValue( TMP_SWITCH ) < 0 then
		drawChronometer()
	end
end

return { init=init, run=run }





